@@ -8,7 +8,7 @@ from nesteddictionary import NestedDict
 
 from .entity_mapper import EntityMapper
 from .fact import Fact
-from .parsers import AddressParser, ContactEmailNamePairParser, ContactEmailParser, ContactPhoneParser
+from .parsers import AddressParser, ContactEmailNamePairParser, ContactEmailParser, ContactPhoneParserHighRecall, ContactPhoneParserHighPrecision, ContactPhoneParserStrict
 from .russian_transliterator import Transliterate
 
 
@@ -27,11 +27,13 @@ class Document:
 
     FACT_NAME_EMAIL = "EMAIL"
     FACT_NAME_ADDRESS = "ADDR"
-    FACT_NAME_PHONE = "PHONE"
+    FACT_NAME_PHONE_HIGH_RECALL = "PHONE_high_recall"
+    FACT_NAME_PHONE_HIGH_PRECISION = "PHONE_high_precision"
+    FACT_NAME_PHONE_STRICT = "PHONE_strict"
     FACT_NAMES_NER = ("PER", "ORG", "GPE", "LOC")
     FACT_NAME_NAMEMAIL = "NAMEMAIL"
 
-    KNOWN_ENTITIES = (FACT_NAME_EMAIL, FACT_NAME_ADDRESS, FACT_NAME_PHONE, FACT_NAME_NAMEMAIL) + FACT_NAMES_NER
+    KNOWN_ENTITIES = (FACT_NAME_EMAIL, FACT_NAME_ADDRESS, FACT_NAME_PHONE_STRICT, FACT_NAME_NAMEMAIL) + FACT_NAMES_NER
 
     CLOSE_FACT_DISTANCE = 150
 
@@ -177,17 +179,32 @@ class Document:
                 )
                 self.__texta_facts.append(new_fact)
 
-
-    def contacts(self):
+    def emails(self):
         text = self.get_words()
         emails = ContactEmailParser(text).parse()
         self.__texta_facts.extend((email.to_fact(Document.FACT_NAME_EMAIL, self.doc_path) for email in emails))
-        phone_numbers = ContactPhoneParser(text, months=self.concat_resources["months"]).parse()
-        self.__texta_facts.extend((number.to_fact(Document.FACT_NAME_PHONE, self.doc_path) for number in phone_numbers))
+
+    def phone_strict(self):
+        text = self.get_words()
+        phone_numbers_strict = ContactPhoneParserStrict(text).parse()
+        self.__texta_facts.extend((number.to_fact(Document.FACT_NAME_PHONE_STRICT, self.doc_path) for number in phone_numbers_strict))
+        
+    def phone_high_recall(self):
+        text = self.get_words()
+        phone_numbers = ContactPhoneParserHighRecall(text, months=self.concat_resources["months"]).parse()
+        self.__texta_facts.extend((number.to_fact(Document.FACT_NAME_PHONE_HIGH_RECALL, self.doc_path) for number in phone_numbers))
+        
+    def phone_high_precision(self):
+        text = self.get_words()
+        phone_numbers_high_precision = ContactPhoneParserHighPrecision(text).parse()
+        self.__texta_facts.extend((number.to_fact(Document.FACT_NAME_PHONE_HIGH_PRECISION, self.doc_path) for number in phone_numbers_high_precision))
+            
+    def addresses(self): 
+        print("Hakkan aadresse otsima")
+        text = self.get_words()
         addresses = AddressParser(text, self.stanza_entities, self.dominant_language_code).parse()
         self.__texta_facts.extend((addr.to_fact(Document.FACT_NAME_ADDRESS, self.doc_path) for addr in addresses))
 
-
     def transliteration(self):
         if self.dominant_language_code in Document.langs_to_transliterate:
             for word in self.stanza_words:
@@ -274,7 +291,6 @@ class Document:
                     )
                     self.__texta_facts.append(new_fact)
 
-
     def namemail(self):
         """
         Find name-email pairs.
@@ -510,9 +526,23 @@ class Document:
                 new_bounds.append(new_fact)
         return new_bounds
 
+    def add_phone_high_recall(self, facts):
+        """
+        remove previous phone facts
+        add phoneparser made for emails (ContactPhoneParserHighRecall)
+        """
+        new_facts = []
+        for fact in facts:
+            if not re.match("PHONE", fact.fact_type):
+                new_facts.append(fact)
+        text = self.get_words()
+        phone_numbers = ContactPhoneParserHighRecall(text, months=self.concat_resources["months"]).parse()
+        new_facts.extend((number.to_fact(Document.FACT_NAME_PHONE_HIGH_RECALL, self.doc_path) for number in phone_numbers))
+        return new_facts
 
     def bounded(self):
         facts = self.remove_duplicate_facts_by_span(self.__texta_facts)
+        facts = self.add_phone_high_recall(facts) #bounded uses only ContactPhoneParserHighRecall results.
         new_bounded = self.bound_close_ones(facts)
         new_bounded = self.remove_overlaping_in_bounded(new_bounded)
         new_bounded = self.concatenate_subset_bounds(new_bounded)
